## Formatted online version of this README
https://www.drupal.org/project/gitinfo

## Summary
An overview in the Status report at _../admin/reports/status_ of fundamental GIT info for all repos, from the project root to the deepest GIT folder it detects. It highlights the things that need attention, in that case resulting in a warning.

Provides per GIT directory (only if relevant):

*   [remote](https://git-scm.com/docs/git-remote "Git - git-remote Documentation") URL, links to the repository
*   [status](https://git-scm.com/docs/git-status "Git - git-status Documentation") (current branch, untracked files and local file changes)
*   current [tag](https://git-scm.com/docs/git-tag "Git - git-tag Documentation")
*   last [pull](https://git-scm.com/docs/git-pull "Git - git-pull Documentation") date
*   last [commit](https://git-scm.com/docs/git-commit "Git - git-commit Documentation") date with the commit hash, links to the commit
*   [ignored](https://git-scm.com/docs/gitignore "Git - gitignore Documentation") or skipped existing folders and files
*   [Git submodules](https://git-scm.com/book/en/v2/Git-Tools-Submodules)
*   list of [stashes](https://git-scm.com/docs/git-stash "Git - git-stash Documentation").

It provides **a quick insight for Drupal projects that contain multiple Git repositories** and/or [Git submodules](https://git-scm.com/book/en/v2/Git-Tools-Submodules "Git - Submodules") which otherwise can only be obtained through the terminal, navigating to each repo and executing many long awkward Git commands yourself each time.

If you find changed files **on your server** it usually means trouble (including hacking attempts) as they should all have been deployed under version control. The _sites/*/files_ folder and similar are in the _.gitignore_ file. It is recommended to have the complete Drupal project under GIT version control to detect any changes. Use `git init` on it if this is not the case. It is usually one level above the Drupal root (the folder named _web_ or _docroot_) and contains apart from that also the project's _composer.json_ file and consequently the _vendor_ folder.

On your **local development** environment changed files indicate you have to stage (with _git add_), commit or push your work. In the Status report, it is all pointed out.

## Requirements

*   GIT should be installed.
*   Unix based OS (Linux or OSX).
*   The [shell_exec()](http://php.net/manual/en/function.shell-exec.php " shell_exec - Manual") PHP function has NOT been disabled in the php.ini. If it is disabled on your server you can still use the module for your local development only.
*   Having at least one folder under Git version control, preferably also the project root.

## Troubleshooting

*   Recently added GIT folders might not be detected right away. Run `sudo updatedb` first to update the Linux 'locate' cache. On OSX: `sudo /usr/libexec/locate.updatedb`.\
    [What's the equivalent of Linux's updatedb command for the Mac? - Super User](https://superuser.com/a/109592/656420).
*   The correct indenting of a nested unordered list depends on the used admin theme. With the default admin theme named Seven, it works well.
*   If no tag exists for a GIT folder then probably branches are used for release management.

## How to define directories outside the project root

By default the project root gets scanned recursively for GIT repos. To track also others add for example the following to the _settings.php_ file or a custom module (comma separated):

    global $_gitinfo_extra_folders_;
    $_gitinfo_extra_folders_ = '/var/www/data,/home/admin/lib/angular';

Note the underscores and that an absolute path should be used (from system root).

Alternatively a Drupal setting/variable can be used. For example add the following to the _settings.php_ file (comma separated):

#### Drupal 8

    $config['gitinfo.settings']['gitinfo_extra_folders'] = '/var/www/data,/home/admin/lib/angular';

#### Drupal 7

    $conf['gitinfo_extra_folders'] = '/var/www/data,/home/admin/lib/angular';

No UI is offered. There is no settings page.

The global variable gets priority as it is a higher-level method (needs commit rights) and would disallow setting it through Drupal settings/variables. That might be preferred.

Note that **defining non-existing directories will result in a warning** in the Status report.

### Alternatively use Drush to set configuration

#### Drupal 8

    drush cset -y gitinfo.settings gitinfo_extra_folders  '/var/www/data,/home/admin/lib/angular'

#### Drupal 7

    drush vset -y gitinfo_extra_folders  '/var/www/data,/home/admin/lib/angular'

## Concept

The module executes several git commands on the current environment and shows the output on the Status report at _../admin/reports/status_. It uses [shell_exec()](http://php.net/manual/en/function.shell-exec.php " shell_exec - Manual") for it (see [command - PHP exec() vs system() vs passthru() - Stack Overflow](https://stackoverflow.com/a/39681338/523688)).

**It shows the info for all git folders it detects recursively.**

### Used commands in the module's code

#### git --version

Shows the current git version (and checks if git is available).

#### locate "$PWD*/.git" | grep -e ".git$" | sort

To create a list of all detected git folders to loop through with below commands.\
[Recursively check for git directory updates - Stack Overflow](https://stackoverflow.com/a/8838668/523688).

First, we use 'locate' instead of 'find' because it returns the full paths of the git folders and is quicker. If a GIT folder does not show immediately, see [How to update Linux "locate" cache - Unix & Linux Stack Exchange](https://unix.stackexchange.com/a/124758/139407).

#### find $PWD -name "*.git" -type d | grep -e ".git$" | sort

As a fallback when 'locate' does not return any results, we use 'find'.

#### git config --get remote.origin.url

[How can I determine the URL that a local Git repository was originally cloned from? - Stack Overflow](https://stackoverflow.com/a/4090938/523688).\
If the remote starts with _git@.._ instead of _https://.._ it indicates it has been cloned through SSH.

#### git status

Shows the current branch, detects untracked files and local file changes. The whole GIT Info section shows in the _Warnings found_ section (D8) or colored yellow (D7) if any of the below texts are found:

*   _Changed but not updated_ (for older Git versions)
*   _Changes not staged for commit_
*   _Changes to be committed_
*   _Your branch is ahead_ (you should push your changes)
*   _Your branch is behind_ (you should pull changes from the repo)
*   _No commits yet_ (a new repo, make an intial commit on it)
*   _HEAD detached_ (the commits you are making do not belong to a branch)
*   _Remote: Not added yet_ (see the _remote_ command above, it makes no sense to have Git only locally)

These texts are then notified by being highlighted in red. If all is fine you should not have any changed files **on your server** as all files are deployed under version control by Git. In that case the GIT Info section shows as OK in the _Checked_ section (D8) or green colored (D7). On your local machine a warning here indicates you have some files to stage (with _git add_), commit or push, or you still should create or add a remote repo URL.

#### git tag --points-at HEAD

Shows all tags on the current HEAD (or commit). If no tag exists on a server environment probably branches are used for release management.\
[git checkout - Show which git tag you are on? - Stack Overflow](https://stackoverflow.com/a/37497511/523688).

#### stat -c %Y .git/FETCH_HEAD

Last pull timestamp. On the Status report as a fuzzy timestamp (time ago) showing the exact date and time on mouse hover.\
[How do I check the date and time of the latest `git pull` that was executed? - Stack Overflow](https://stackoverflow.com/a/9229377/523688).

#### git log -1 --format="%at⁏%an⁏%ae⁏%cn⁏%ce⁏%h⁏%s"

The last commit message together with the author, committer and on the Status report a fuzzy timestamp (time ago) showing the exact date and time on mouse hover.\
[How to read last commit comment? - Stack Overflow](https://stackoverflow.com/a/7293026/523688).

#### git clean -ndX

List of ignored existing folders and files.\
[Git command to show which specific files are ignored by .gitignore - Stack Overflow](https://stackoverflow.com/a/2196755/523688).

#### git submodule status | cut -d\' \' -f3-4

Shows the detected submodules of the current repo.\
[List submodules in a Git repository - Stack Overflow](https://stackoverflow.com/a/19659265/523688).

#### git stash list

List the stash entries that you currently have.\
[Git - git-stash Documentation](https://git-scm.com/docs/git-stash)

Note that git stashes only exist locally and will normally not get pushed. **Having stashes will result in a warning just as a reminder**, suggesting to 'pop', 'drop' or 'clear'.\
[Is git stash stack pushed to the remote repo? - Stack Overflow](https://stackoverflow.com/a/23952704/523688).

## Setting it up only on development

It makes perfect sense to have the module enabled on all environments. The module originated from the need to check if code changes were made directly on the production server (they shouldn't).

If, however, you prefer to use it only for development, you need to:

*   add it to Composer with `composer require drupal/gitinfo --dev`
*   use on production always `composer install --no-dev` ([recommended](https://www.drupal.org/docs/8/install/step-2-install-dependencies-with-composer " Install dependencies with composer | Drupal 8 guide on Drupal.org"))
*   follow the instructions described in: [Creating a simple split configuration: Dev modules only in dev environments | Drupal 8 guide on Drupal.org](https://www.drupal.org/docs/8/modules/configuration-split/creating-a-simple-split-configuration-dev-modules-only-in-dev)
